"""Imports for Python API.

This file is MACHINE GENERATED! Do not edit.
Generated by: tensorflow/tools/api/generator/create_python_api.py script.
"""
from tensorflow.tools.api.generator.api.keras.datasets import boston_housing
from tensorflow.tools.api.generator.api.keras.datasets import cifar10
from tensorflow.tools.api.generator.api.keras.datasets import cifar100
from tensorflow.tools.api.generator.api.keras.datasets import fashion_mnist
from tensorflow.tools.api.generator.api.keras.datasets import imdb
from tensorflow.tools.api.generator.api.keras.datasets import mnist
from tensorflow.tools.api.generator.api.keras.datasets import reuters