"""Imports for Python API.

This file is MACHINE GENERATED! Do not edit.
Generated by: tensorflow/tools/api/generator/create_python_api.py script.
"""
from tensorflow.python import constant_initializer as Constant
from tensorflow.python import ones_initializer as Ones
from tensorflow.python import orthogonal_initializer as Orthogonal
from tensorflow.python import random_normal_initializer as RandomNormal
from tensorflow.python import random_uniform_initializer as RandomUniform
from tensorflow.python import truncated_normal_initializer as TruncatedNormal
from tensorflow.python import variance_scaling_initializer as VarianceScaling
from tensorflow.python import zeros_initializer as Zeros
from tensorflow.python.keras._impl.keras.initializers import Identity
from tensorflow.python.keras._impl.keras.initializers import Initializer
from tensorflow.python.keras._impl.keras.initializers import deserialize
from tensorflow.python.keras._impl.keras.initializers import get
from tensorflow.python.keras._impl.keras.initializers import glorot_normal
from tensorflow.python.keras._impl.keras.initializers import glorot_uniform
from tensorflow.python.keras._impl.keras.initializers import he_normal
from tensorflow.python.keras._impl.keras.initializers import he_uniform
from tensorflow.python.keras._impl.keras.initializers import lecun_normal
from tensorflow.python.keras._impl.keras.initializers import lecun_uniform
from tensorflow.python.keras._impl.keras.initializers import serialize