"""Imports for Python API.

This file is MACHINE GENERATED! Do not edit.
Generated by: tensorflow/tools/api/generator/create_python_api.py script.
"""
from tensorflow.python.training.queue_runner import QueueRunner
from tensorflow.python.training.queue_runner import add_queue_runner
from tensorflow.python.training.queue_runner import start_queue_runners