"""Imports for Python API.

This file is MACHINE GENERATED! Do not edit.
Generated by: tensorflow/tools/api/generator/create_python_api.py script.
"""
from tensorflow.python.util.compat import as_bytes
from tensorflow.python.util.compat import as_bytes as as_str
from tensorflow.python.util.compat import as_str_any
from tensorflow.python.util.compat import as_text
from tensorflow.python.util.compat import bytes_or_text_types
from tensorflow.python.util.compat import complex_types
from tensorflow.python.util.compat import integral_types
from tensorflow.python.util.compat import path_to_str
from tensorflow.python.util.compat import real_types