"""Imports for Python API.

This file is MACHINE GENERATED! Do not edit.
Generated by: tensorflow/tools/api/generator/create_python_api.py script.
"""
from tensorflow.python import constant_initializer as constant
from tensorflow.python import global_variables_initializer as global_variables
from tensorflow.python import local_variables_initializer as local_variables
from tensorflow.python import ones_initializer as ones
from tensorflow.python import orthogonal_initializer as orthogonal
from tensorflow.python import random_normal_initializer as random_normal
from tensorflow.python import random_uniform_initializer as random_uniform
from tensorflow.python import truncated_normal_initializer as truncated_normal
from tensorflow.python import uniform_unit_scaling_initializer as uniform_unit_scaling
from tensorflow.python import variables_initializer as variables
from tensorflow.python import variance_scaling_initializer as variance_scaling
from tensorflow.python import zeros_initializer as zeros
from tensorflow.python.keras._impl.keras.initializers import Identity as identity